#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import io
import copy
import stat
import struct
import sys
import unittest
import subprocess
import tempfile
import urllib
import shutil

import rosinstall
import rosinstall.helpers

def _add_to_file(path, content):
     f = io.open(path, 'a')
     f.write(content)
     f.close()

ROSINSTALL_FN = [os.path.join(os.getcwd(), 'scripts/rosinstall')]


class AbstractSCMTest(unittest.TestCase):
    def setUp(self, scm):


        self.test_root_path = tempfile.mkdtemp()
        self.directories = {"root":self.test_root_path}

        self.new_environ = os.environ
        self.new_environ["PYTHONPATH"] = os.path.join(os.getcwd(), "src")

        
        # setup fake ros root
        self.ros_path = os.path.join(self.test_root_path, "ros")
        os.makedirs(self.ros_path)
        _add_to_file(os.path.join(self.ros_path, "stack.xml"), u'<stack></stack>')
        _add_to_file(os.path.join(self.ros_path, "setup.sh"), u'export ROS_ROOT=`pwd`')
        self.local_path = os.path.join(self.test_root_path, "ws")
        os.makedirs(self.local_path)

    def tearDown(self):
        for d in self.directories:
            shutil.rmtree(self.directories[d])        
        
class RosinstallDiffGitTest(AbstractSCMTest):

    def setUp(self):
        AbstractSCMTest.setUp(self, "git")
        remote_path = os.path.join(self.test_root_path, "remote")
        os.makedirs(remote_path)
        
        # create a "remote" repo
        subprocess.check_call(["git", "init"], cwd=remote_path)
        subprocess.check_call(["touch", "fixed.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified-fs.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted-fs.txt"], cwd=remote_path)
        subprocess.check_call(["git", "add", "*"], cwd=remote_path)
        subprocess.check_call(["git", "commit", "-m", "modified"], cwd=remote_path)

        # rosinstall the remote repo and fake ros
        _add_to_file(os.path.join(self.local_path, ".rosinstall"), u"- other: {local-name: ../ros}\n- git: {local-name: clone, uri: remote}")

        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "-n"])
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        clone_path = os.path.join(self.local_path, "clone")
        
        # make local modifications
        subprocess.check_call(["rm", "deleted-fs.txt"], cwd=clone_path)
        subprocess.check_call(["git", "rm", "deleted.txt"], cwd=clone_path)
        _add_to_file(os.path.join(clone_path, "modified-fs.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "modified.txt"), u"foo\n")
        subprocess.check_call(["git", "add", "modified.txt"], cwd=clone_path)
        _add_to_file(os.path.join(clone_path, "added-fs.txt"), u"tada\n")
        _add_to_file(os.path.join(clone_path, "added.txt"), u"flam\n")
        subprocess.check_call(["git", "add", "added.txt"], cwd=clone_path)
                
    def check_diff_output(self, output):
        # sha ids are always same with git
        self.assertEqual('diff --git clone/added.txt clone/added.txt\nnew file mode 100644\nindex 0000000..8d63207\n--- /dev/null\n+++ clone/added.txt\n@@ -0,0 +1 @@\n+flam\ndiff --git clone/deleted-fs.txt clone/deleted-fs.txt\ndeleted file mode 100644\nindex e69de29..0000000\ndiff --git clone/deleted.txt clone/deleted.txt\ndeleted file mode 100644\nindex e69de29..0000000\ndiff --git clone/modified-fs.txt clone/modified-fs.txt\nindex e69de29..257cc56 100644\n--- clone/modified-fs.txt\n+++ clone/modified-fs.txt\n@@ -0,0 +1 @@\n+foo\ndiff --git clone/modified.txt clone/modified.txt\nindex e69de29..257cc56 100644\n--- clone/modified.txt\n+++ clone/modified.txt\n@@ -0,0 +1 @@\n+foo\n\n', output)
        
    def test_Rosinstall_diff_git_outside(self):
        """Test diff output for git when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.check_diff_output(output)
        

    def test_Rosinstall_diff_git_inside(self):
        """Test diff output for git when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.check_diff_output(output)
        
    def test_Rosinstall_status_git_inside(self):
        """Test status output for git when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--status"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.assertEqual('A       clone/added.txt\n D      clone/deleted-fs.txt\nD       clone/deleted.txt\n M      clone/modified-fs.txt\nM       clone/modified.txt\n\n', output)
   
    def test_Rosinstall_status_git_outside(self):
        """Test status output for git when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.assertEqual('A       clone/added.txt\n D      clone/deleted-fs.txt\nD       clone/deleted.txt\n M      clone/modified-fs.txt\nM       clone/modified.txt\n\n', output)

    def test_Rosinstall_status_git_untracked(self):
        """Test untracked status output for git when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status-untracked"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('A       clone/added.txt\n D      clone/deleted-fs.txt\nD       clone/deleted.txt\n M      clone/modified-fs.txt\nM       clone/modified.txt\n??      clone/added-fs.txt\n\n', output)


class RosinstallDiffHgTest(AbstractSCMTest):

    def setUp(self):
        AbstractSCMTest.setUp(self, "hg")
        remote_path = os.path.join(self.test_root_path, "remote")
        os.makedirs(remote_path)
        
        # create a "remote" repo
        subprocess.check_call(["hg", "init"], cwd=remote_path)
        subprocess.check_call(["touch", "fixed.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified-fs.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted-fs.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "add", "fixed.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "add", "modified.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "add", "modified-fs.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "add", "deleted.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "add", "deleted-fs.txt"], cwd=remote_path)
        subprocess.check_call(["hg", "commit", "-m", "modified"], cwd=remote_path)

        # rosinstall the remote repo and fake ros
        _add_to_file(os.path.join(self.local_path, ".rosinstall"), u"- other: {local-name: ../ros}\n- hg: {local-name: clone, uri: remote}")

        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "-n"])
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        clone_path = os.path.join(self.local_path, "clone")
        
        # make local modifications
        subprocess.check_call(["rm", "deleted-fs.txt"], cwd=clone_path)
        subprocess.check_call(["hg", "rm", "deleted.txt"], cwd=clone_path)
        _add_to_file(os.path.join(clone_path, "modified-fs.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "modified.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "added-fs.txt"), u"tada\n")
        _add_to_file(os.path.join(clone_path, "added.txt"), u"flam\n")
        subprocess.check_call(["hg", "add", "added.txt"], cwd=clone_path)
        
    def check_diff_output(self, output):
        # sha ids are always same with hg
        self.assertEqual('diff --git clone/added.txt clone/added.txt\nnew file mode 100644\n--- /dev/null\n+++ clone/added.txt\n@@ -0,0 +1,1 @@\n+flam\ndiff --git clone/deleted.txt clone/deleted.txt\ndeleted file mode 100644\ndiff --git clone/modified-fs.txt clone/modified-fs.txt\n--- clone/modified-fs.txt\n+++ clone/modified-fs.txt\n@@ -0,0 +1,1 @@\n+foo\ndiff --git clone/modified.txt clone/modified.txt\n--- clone/modified.txt\n+++ clone/modified.txt\n@@ -0,0 +1,1 @@\n+foo\n\n\n', output)
        
    def test_Rosinstall_diff_hg_outside(self):
        """Test diff output for hg when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.check_diff_output(output)

    def test_Rosinstall_diff_hg_inside(self):
        """Test diff output for hg when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.check_diff_output(output)
        
        
    def test_Rosinstall_status_hg_inside(self):
        """Test status output for hg when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--status"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.assertEqual('M       clone/modified-fs.txt\nM       clone/modified.txt\nA       clone/added.txt\nR       clone/deleted.txt\n!       clone/deleted-fs.txt\n\n', output)
   
    def test_Rosinstall_status_hg_outside(self):
        """Test status output for hg when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('M       clone/modified-fs.txt\nM       clone/modified.txt\nA       clone/added.txt\nR       clone/deleted.txt\n!       clone/deleted-fs.txt\n\n', output)

    def test_Rosinstall_status_hg_untracked(self):
        """Test untracked status output for hg when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status-untracked"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('M       clone/modified-fs.txt\nM       clone/modified.txt\nA       clone/added.txt\nR       clone/deleted.txt\n!       clone/deleted-fs.txt\n?       clone/added-fs.txt\n\n', output)

class RosinstallDiffBzrTest(AbstractSCMTest):

    def setUp(self):
        AbstractSCMTest.setUp(self, "bzr")
        remote_path = os.path.join(self.test_root_path, "remote")
        os.makedirs(remote_path)
        
        # create a "remote" repo
        subprocess.check_call(["bzr", "init"], cwd=remote_path)
        subprocess.check_call(["touch", "fixed.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "modified-fs.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted.txt"], cwd=remote_path)
        subprocess.check_call(["touch", "deleted-fs.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "add", "fixed.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "add", "modified.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "add", "modified-fs.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "add", "deleted.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "add", "deleted-fs.txt"], cwd=remote_path)
        subprocess.check_call(["bzr", "commit", "-m", "modified"], cwd=remote_path)

        # rosinstall the remote repo and fake ros
        _add_to_file(os.path.join(self.local_path, ".rosinstall"), u"- other: {local-name: ../ros}\n- bzr: {local-name: clone, uri: remote}")

        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "-n"])
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE, env=self.new_environ)
        output=call.communicate()[0]

        clone_path = os.path.join(self.local_path, "clone")
        
        # make local modifications
        subprocess.check_call(["rm", "deleted-fs.txt"], cwd=clone_path)
        subprocess.check_call(["bzr", "rm", "deleted.txt"], cwd=clone_path)
        _add_to_file(os.path.join(clone_path, "modified-fs.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "modified.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "added-fs.txt"), u"tada\n")
        _add_to_file(os.path.join(clone_path, "added.txt"), u"flam\n")
        subprocess.check_call(["bzr", "add", "added.txt"], cwd=clone_path)
        
    def check_diff_output (self, output):
        # uncomment following line for easiest way to get actual output with escapes
        # self.assertEqual(None, output);

        # bzr writes date-time of file into diff
        self.assertTrue(output.startswith("=== added file 'added.txt'\n--- clone/added.txt"), msg=0)
        self.assertTrue(0 < output.find("+++ clone/added.txt"), msg=1)
        self.assertTrue(0 < output.find("@@ -0,0 +1,1 @@\n+flam\n\n"), msg=2)
        self.assertTrue(0 < output.find("=== removed file 'deleted-fs.txt'\n=== removed file 'deleted.txt'\n=== modified file 'modified-fs.txt'\n--- clone/modified-fs.txt"), msg=3 )
        self.assertTrue(0 < output.find("@@ -0,0 +1,1 @@\n+foo\n\n=== modified file 'modified.txt'\n--- clone/modified.txt"), msg=4)
    
    def test_Rosinstall_diff_bzr_outside(self):
        """Test diff output for bzr when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.check_diff_output(output)


    def test_Rosinstall_diff_bzr_inside(self):
        """Test diff output for bzr when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.check_diff_output(output)

        
    def test_Rosinstall_status_bzr_inside(self):
        """Test status output for bzr when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--status"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.assertEqual('+N      clone/added.txt\n D      clone/deleted-fs.txt\n-D      clone/deleted.txt\n M      clone/modified-fs.txt\n M      clone/modified.txt\n\n', output)
   
    def test_Rosinstall_status_bzr_outside(self):
        """Test status output for bzr when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('+N      clone/added.txt\n D      clone/deleted-fs.txt\n-D      clone/deleted.txt\n M      clone/modified-fs.txt\n M      clone/modified.txt\n\n', output)

    def test_Rosinstall_status_bzr_untracked(self):
        """Test status output for bzr when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status-untracked"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('?       clone/added-fs.txt\n+N      clone/added.txt\n D      clone/deleted-fs.txt\n-D      clone/deleted.txt\n M      clone/modified-fs.txt\n M      clone/modified.txt\n\n', output)


        
class RosinstallDiffSvnTest(AbstractSCMTest):

    def setUp(self):
        AbstractSCMTest.setUp(self, "svn")
        remote_path = os.path.join(self.test_root_path, "remote")
        filler_path = os.path.join(self.test_root_path, "filler")
        svn_uri = "file://localhost"+remote_path
        
        # create a "remote" repo
        subprocess.check_call(["svnadmin", "create", remote_path], cwd=self.test_root_path)
        subprocess.check_call(["svn", "checkout", svn_uri, filler_path], cwd=self.test_root_path)   
        subprocess.check_call(["touch", "fixed.txt"], cwd=filler_path)
        subprocess.check_call(["touch", "modified.txt"], cwd=filler_path)
        subprocess.check_call(["touch", "modified-fs.txt"], cwd=filler_path)
        subprocess.check_call(["touch", "deleted.txt"], cwd=filler_path)
        subprocess.check_call(["touch", "deleted-fs.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "add", "fixed.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "add", "modified.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "add", "modified-fs.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "add", "deleted.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "add", "deleted-fs.txt"], cwd=filler_path)
        subprocess.check_call(["svn", "commit", "-m", "modified"], cwd=filler_path)

        # rosinstall the remote repo and fake ros
        _add_to_file(os.path.join(self.local_path, ".rosinstall"), u"- other: {local-name: ../ros}\n- svn: {local-name: clone, uri: '"+svn_uri+"'}")

        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "-n"])
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE, env=self.new_environ)
        output=call.communicate()[0]
        clone_path = os.path.join(self.local_path, "clone")


        # make local modifications
        subprocess.check_call(["rm", "deleted-fs.txt"], cwd=clone_path)
        subprocess.check_call(["svn", "rm", "deleted.txt"], cwd=clone_path)
        
        #_add_to_file(os.path.join(clone_path, "modified-fs.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "modified.txt"), u"foo\n")
        _add_to_file(os.path.join(clone_path, "added-fs.txt"), u"tada\n")
        _add_to_file(os.path.join(clone_path, "added.txt"), u"flam\n")
        subprocess.check_call(["svn", "add", "added.txt"], cwd=clone_path)
        
     
    def check_diff_output(self, output):
        #self.assertEqual('Index: clone/added.txt\n===================================================================\n--- clone/added.txt\t(revision 0)\n+++ clone/added.txt\t(revision 0)\n@@ -0,0 +1 @@\n+flam\n\nProperty changes on: clone/added.txt\n===================================================================\nAdded: svn:eol-style\n   + native\n\nIndex: clone/modified.txt\n===================================================================\n--- clone/modified.txt\t(revision 1)\n+++ clone/modified.txt\t(working copy)\n@@ -0,0 +1 @@\n+foo\n\n', output)
         self.assertEqual('Index: clone/added.txt\n===================================================================\n--- clone/added.txt\t(revision 0)\n+++ clone/added.txt\t(revision 0)\n@@ -0,0 +1 @@\n+flam\nIndex: clone/modified.txt\n===================================================================\n--- clone/modified.txt\t(revision 1)\n+++ clone/modified.txt\t(working copy)\n@@ -0,0 +1 @@\n+foo\n\n', output)
         
        
    def test_Rosinstall_diff_svn_outside(self):
        """Test diff output for svn when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.check_diff_output(output)

    def test_Rosinstall_diff_svn_inside(self):
        """Test diff output for svn when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--diff"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.check_diff_output(output)

    def test_Rosinstall_status_svn_inside(self):
        """Test status output for svn when run inside workspace"""
        # dir created by make
        directory = self.test_root_path + "/ws"
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend([".", "--status"])
        
        call = subprocess.Popen(cmd, cwd=directory, stdout=subprocess.PIPE)
        output=call.communicate()[0]
        self.assertEqual('A       clone/added.txt\nD       clone/deleted.txt\n!       clone/deleted-fs.txt\nM       clone/modified.txt\n\n', output)
   
    def test_Rosinstall_status_svn_outside(self):
        """Test status output for svn when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('A       clone/added.txt\nD       clone/deleted.txt\n!       clone/deleted-fs.txt\nM       clone/modified.txt\n\n', output)

    def test_Rosinstall_status_svn_untracked(self):
        """Test status output for svn when run outside workspace"""
        # dir created by make
        cmd = copy.copy(ROSINSTALL_FN)
        cmd.extend(["ws", "--status-untracked"])
        
        call = subprocess.Popen(cmd, cwd=self.test_root_path, stdout=subprocess.PIPE)
        output=call.communicate()[0]

        self.assertEqual('?       clone/added-fs.txt\nA       clone/added.txt\nD       clone/deleted.txt\n!       clone/deleted-fs.txt\nM       clone/modified.txt\n\n', output)




# Software License Agreement (BSD License)
#
# Copyright (c) 2010, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import urlparse
import urllib2
import yaml
import subprocess
import sys

def conditional_abspath(uri):
  """
  @param uri: The uri to check
  @return: abspath(uri) if local path otherwise pass through uri
  """
  u = urlparse.urlparse(uri)
  if u.scheme == '': # maybe it's a local file?
    return os.path.abspath(uri)
  else:
    return uri

def is_path_stack(path):
  """
  
  @return: True if the path provided is the root of a stack.
  """
  stack_path = os.path.join(path,'stack.xml')
  if os.path.isfile(stack_path):
    return True
  return False

def is_path_ros(path):
  """
  warning: exits with code 1 if stack document is invalid
  @param path: path of directory to check
  @type  path: str
  @return: True if path points to the ROS stack
  @rtype: bool
  """
  stack_path = os.path.join(path,'stack.xml')
  if os.path.isfile(stack_path):
    return 'ros' == os.path.basename(path)
  return False


def get_yaml_from_uri(uri):

  # now that we've got a config uri and a path, let's move out.
  f = 0
  if os.path.isfile(uri):
    try:
      f = open(uri, 'r')
    except IOError as e:
      sys.stderr.write("error opening file [%s]: %s\n" % (uri, e))
      return None
  else:
    try:
      f = urllib2.urlopen(uri)
    except IOError as e:
      sys.stderr.write("Is not a local file, nor able to download as a URL [%s]: %s\n" % (uri, e))
    except ValueError as e:
      sys.stderr.write("Is not a local file, nor a valid URL [%s] : %s\n" % (uri,e))
  if not f:
    sys.stderr.write("couldn't load config uri %s\n" % uri)
    return None
  try:
    y = yaml.load(f);
  except yaml.YAMLError as e:
    sys.stderr.write("Invalid rosinstall yaml format in [%s]: %s\n" % (uri, e ))
    return None
  return y
  



#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import stat
import struct
import sys
import unittest
import subprocess
import tempfile
import urllib
import shutil

import rosinstall
import rosinstall.helpers

class RosinstallCommandlineTest(unittest.TestCase):

    def setUp(self):
        #self.rosinstall_tempfile = tempfile.NamedTemporaryFile(mode='a+b')
        self.rosinstall_fn = ["./scripts/rosinstall", "-n"]
        self.new_environ = os.environ
        self.new_environ["PYTHONPATH"] = os.path.join(os.getcwd(), "src")
        #urllib.urlretrieve("https://code.ros.org/svn/ros/installers/trunk/rosinstall/rosinstall2", self.rosinstall_fn)
        #os.chmod(self.rosinstall_fn, stat.S_IRWXU)
        self.directories = {}


    def tearDown(self):
        for d in self.directories:
            shutil.rmtree(self.directories[d])
        #os.remove(self.rosinstall_fn)

    def test_Rosinstall_executable(self):
        cmd = self.rosinstall_fn
        print cmd.append("-h")
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

    def test_Rosinstall_ros(self):
        directory = tempfile.mkdtemp()
        self.directories["ros"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "ros_w_release.rosinstall")])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))
        shutil.rmtree(directory)
        self.directories.pop("ros")

    def DISABLED_Rosinstall_ros_stack(self):
        directory = tempfile.mkdtemp()
        self.directories["stack"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "distro_stack.rosinstall")])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))
        shutil.rmtree(directory)
        self.directories.pop("stack")

    def DISABLED_Rosinstall_ros_variant(self):
        directory = tempfile.mkdtemp()
        self.directories["variant"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "distro_variant.rosinstall")])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))
        shutil.rmtree(directory)
        self.directories.pop("variant")


class RosinstallCommandlineOverlays(unittest.TestCase):

    def setUp(self):
        self.rosinstall_tempfile = tempfile.NamedTemporaryFile(mode='a+b')
        self.rosinstall_fn = ["./scripts/rosinstall", "-n"]
        self.new_environ = os.environ
        self.new_environ["PYTHONPATH"] = os.path.join(os.getcwd(), "src")

        #self.rosinstall_fn = "/tmp/test_rosinstall_temp_version"
        #urllib.urlretrieve("https://code.ros.org/svn/ros/installers/trunk/rosinstall/rosinstall", self.rosinstall_fn)
        #os.chmod(self.rosinstall_fn, stat.S_IRWXU)
        self.directories = {}
        self.base = tempfile.mkdtemp()
        cmd = self.rosinstall_fn[:]
        #cmd.extend([self.base, "http://www.ros.org/rosinstalls/boxturtle_base.rosinstall"])
        cmd.extend([self.base, os.path.join("test", "rosinstalls", "ros_w_release.rosinstall")])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))


    def tearDown(self):
        for d in self.directories:
            shutil.rmtree(self.directories[d])
        shutil.rmtree(self.base)
        #os.remove(self.rosinstall_fn)

    def test_Rosinstall_rosinstall_file_generation(self):
        generated_rosinstall_filename = os.path.join(self.base, ".rosinstall")
        self.assertTrue(os.path.exists(generated_rosinstall_filename))
        source_yaml = rosinstall.helpers.get_yaml_from_uri(generated_rosinstall_filename)
        self.assertEqual(source_yaml, 
                         [{'svn': { 'uri': 'https://code.ros.org/svn/ros/stacks/ros/tags/boxturtle',
                                    'local-name': 'ros'} },
                          {'svn': { 'uri': 'https://code.ros.org/svn/ros/stacks/ros_release/trunk',
                                    'local-name': 'ros_release'} }
                          ])

        
            

    def test_Rosinstall_common_msgs_as_explicit_overlay(self):
        directory = tempfile.mkdtemp()
        with tempfile.NamedTemporaryFile() as ri_file:
            file_text = """
- other:
    local-name: %s/ros_release
- other:
    local-name: %s/ros
- svn:
    uri: https://code.ros.org/svn/ros-pkg/stacks/common_msgs/tags/boxturtle
    local-name: stacks/common_msgs
"""%(self.base, self.base)
            print file_text
            ri_file.write(file_text)
            ri_file.flush()
                          
            self.directories["tutorials"] = directory
            cmd = self.rosinstall_fn[:]
            cmd.extend([directory, ri_file.name])
            print "EXECUTING", cmd
            self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("tutorials")

    def test_Rosinstall_ros_tutorial_as_argument(self):
        directory = tempfile.mkdtemp()
        self.directories["tutorials2"] = directory
        cmd = self.rosinstall_fn[:]
        cmd.extend([directory, self.base, os.path.join("test", "rosinstalls", "overlay.rosinstall")])
        print "Running command %s"%cmd
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))


        shutil.rmtree(directory)
        self.directories.pop("tutorials2")

class RosinstallOptionsTest(unittest.TestCase):

    def setUp(self):
        #self.rosinstall_tempfile = tempfile.NamedTemporaryFile(mode='a+b')
        self.rosinstall_fn = ["scripts/rosinstall"]
        self.new_environ = os.environ
        self.new_environ["PYTHONPATH"] = os.path.join(os.getcwd(), "src")

        #urllib.urlretrieve("https://code.ros.org/svn/ros/installers/trunk/rosinstall/rosinstall2", self.rosinstall_fn)
        #os.chmod(self.rosinstall_fn, stat.S_IRWXU)
        self.directories = {}


    def tearDown(self):
        for d in self.directories:
            shutil.rmtree(self.directories[d])
        #os.remove(self.rosinstall_fn)


    def test_rosinstall_delete_changes(self):
        directory = tempfile.mkdtemp()
        self.directories["delete"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple.rosinstall"), "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple_changed_uri.rosinstall"), "--delete-changed-uri", "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("delete")

    def test_rosinstall_abort_changes(self):
        directory = tempfile.mkdtemp()
        self.directories["abort"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple.rosinstall"), "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple_changed_uri.rosinstall"), "--abort-changed-uri", "-n"])
        self.assertEqual(1,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("abort")

    def test_rosinstall_backup_changes(self):
        directory = tempfile.mkdtemp()
        self.directories["backup"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple.rosinstall"), "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        directory1 = tempfile.mkdtemp()
        self.directories["backup1"] = directory1
        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple_changed_uri.rosinstall"), "--backup-changed-uri=%s"%directory1, "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        self.assertEqual(len(os.listdir(directory1)), 1)

        shutil.rmtree(directory)
        self.directories.pop("backup")
        shutil.rmtree(directory1)
        self.directories.pop("backup1")


    def test_rosinstall_invalid_fail(self):
        directory = tempfile.mkdtemp()
        self.directories["invalid_fail"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "broken.rosinstall"), "-n"])
        self.assertEqual(1,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("invalid_fail")

    def test_rosinstall_invalid_continue(self):
        directory = tempfile.mkdtemp()
        self.directories["invalid_continue"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "broken.rosinstall"), "-n", "--continue-on-error"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("invalid_continue")

    def test_rosinstall_change_vcs_type(self):
        directory = tempfile.mkdtemp()
        self.directories["change_vcs_type"] = directory
        cmd = self.rosinstall_fn
        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple.rosinstall"), "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        cmd.extend([directory, os.path.join("test", "rosinstalls", "simple_changed_vcs_type.rosinstall"), "--delete-changed-uri", "-n"])
        self.assertEqual(0,subprocess.call(cmd, env=self.new_environ))

        shutil.rmtree(directory)
        self.directories.pop("change_vcs_type")





# Software License Agreement (BSD License)
#
# Copyright (c) 2010, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author: kwc

import os
import sys
import yaml
import urllib2

BRANCH_RELEASE = 'release'
BRANCH_DEVEL = 'devel'

class InvalidData(Exception): pass

def get_rosinstall(name, data, type_, branch=None, prefix=None):
    """
    Compute a rosinstall fragment for checkout
    
    @param name: resource name
    @param data: manifest data for resource
    @param branch: source branch type ('devel' or 'release')
    @param prefix: checkout filepath prefix
    @raise InvalidData
    """
    
    if not 'rosinstall' in data:
        raise InvalidData("rosinstall control information for %s %s\n"%(type_, name))

    ri_entry = None
    if branch and 'rosinstalls' in data:
        ri_entry = data['rosinstalls'].get(branch, None)

    # if we were unable to compute the rosinstall info based on a
    # desired branch, use the default info instead
    if ri_entry is None:
        if data['vcs'] == 'svn':
            # fancy logic to enable package-specific checkout and also
            # fix a bug in the indexer.
            ri_entry = {'svn': {'local-name': name, 'uri': data['vcs_uri']}}
        else:
            ri_entry = data['rosinstall']
        
    if len(ri_entry) != 1:
        raise InvalidData("rosinstall malformed for %s %s\n"%(type_, name))

    prefix = prefix or ''
    for k, v in ri_entry.iteritems():
        if 'local-name' in v:
            local_name = v['local-name']                
            # 3513
            # compute path: we can't use os.path.join because rosinstall paths
            # are always Unix-style.
            paths = [x for x in (prefix, local_name) if x]
            path = '/'.join(paths)
            v['local-name'] = path

    return yaml.dump([ri_entry], default_flow_style=False)

def get_vcs_uri_for_branch(data, branch=None):
    """
    @param data: rosdoc manifest data
    @param branch: source branch type ('devel' or 'release')
    """
    ri_entry = None
    if branch and 'rosinstalls' in data:
        ri_entry = data['rosinstalls'].get(branch, None)
        vcs_type = ri_entry.keys()[0]
        return ri_entry[vcs_type]['uri']
    else:
        return data.get('vcs_uri', '')

def get_vcs(name, data, type_):
    """
    @param name: resource name
    @param data: rosdoc manifest data
    @param type_: resource type ('stack' or 'package')
    """
    return data.get('vcs', '')

def get_repo(name, data, type_):
    """
    @param name: resource name
    @param data: rosdoc manifest data
    @param type_: resource type ('stack' or 'package')
    """
    return data.get('repository', '')

def get_www(name, data, type_):
    """
    @param name: resource name
    @param data: rosdoc manifest data
    @param type_: resource type ('stack' or 'package')
    """
    return data.get('url', '')

def get_rosdoc_manifest(arg, distro_name=None):
    """
    Get the rosdoc manifest data and type of arg.

    @param arg: name of package/stack to get manifest information for.
    get_manifest() gives stacks symbols precedence over package
    symbols.
    @type  arg: str
    
    @return: (manifest data, 'package'|'stack'). 
    @rtype: ({str: str}, str)
    @raise IOError: if data cannot be loaded
    """
    try:
        if distro_name is not None:
            url = 'http://ros.org/doc/%s/api/%s/stack.yaml'%(distro_name, arg)
        else:
            url = 'http://ros.org/doc/api/%s/stack.yaml'%(arg)
        r = urllib2.urlopen(url)
        return yaml.load(r), 'stack'
    except:
        try:
            if distro_name is not None:
                url = 'http://ros.org/doc/%s/api/%s/manifest.yaml'%(distro_name, arg)
            else:
                url = 'http://ros.org/doc/api/%s/manifest.yaml'%(arg)
            r = urllib2.urlopen(url)
            return yaml.load(r), 'package'
        except:
            raise IOError(arg)
        



#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


import os
import stat
import struct
import sys
import unittest

import rosinstall.helpers


class ConditionalAbspath(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_conditional_abspath(self):
        path = "foo"
        self.assertEqual(os.path.normpath(os.path.join(os.getcwd(), path)), rosinstall.helpers.conditional_abspath(path))
        
    def test_is_path_stack(self):
        self.assertTrue(rosinstall.helpers.is_path_stack(os.path.join("test", "example_dirs", "ros")))
        self.assertFalse(rosinstall.helpers.is_path_stack(os.path.join("test", "example_dirs", "roscpp")))

    def test_is_path_ros(self):
        self.assertTrue(rosinstall.helpers.is_path_stack((os.path.join("test", "example_dirs", "ros"))))
        self.assertFalse(rosinstall.helpers.is_path_stack((os.path.join("test", "example_dirs", "roscpp"))))

    def test_get_yaml_from_uri_from_file(self):
        file = os.path.join("test", "example.yaml")
        y = rosinstall.helpers.get_yaml_from_uri(file)
        
        self.assertTrue("text" in y)
        self.assertTrue(y["text"] == "foobar")

        self.assertTrue("number" in y)
        self.assertTrue(y["number"] == 2)

    def test_get_yaml_from_uri_from_missing_file(self):
        file = "/asdfasdfasdfasfasdf_does_not_exist"
        y = rosinstall.helpers.get_yaml_from_uri(file)
        self.assertEqual(y, None)

#TODO Fix this
#    def test_get_yaml_from_uri_from_non_yaml_file(self):
#        file = os.path.join(roslib.packages.get_pkg_dir("test_rosinstall"), "Makefile")
#        y = rosinstall.helpers.get_yaml_from_uri(file)
#        self.assertEqual(y, None)

    def test_get_yaml_from_uri_from_url(self):
        url = "http://www.ros.org/rosinstalls/boxturtle_base.rosinstall"
        y = rosinstall.helpers.get_yaml_from_uri(url)
        
        ros_found = False
        for e in y:
            if "svn" in e:
                element = e["svn"]
                if "local-name" in element:
                    if element["local-name"] == "ros":
                        ros_found = True
        self.assertTrue(ros_found)

    def test_get_yaml_from_uri_from_invalid_url(self):
        url = "http://www.ros.org/invalid"
        y = rosinstall.helpers.get_yaml_from_uri(url)
        self.assertEqual(y, None)




from setuptools import setup

setup(name='rosinstall',
      version= '0.5.24',
      packages=['rosinstall'],
      package_dir = {'':'src'},
      install_requires=['vcstools'],
      scripts = ["scripts/rosinstall", "scripts/roslocate", "scripts/rosws", "scripts/rosco"],
      author = "Tully Foote", 
      author_email = "tfoote@willowgarage.com",
      url = "http://www.ros.org/wiki/rosinstall",
      download_url = "http://pr.willowgarage.com/downloads/rosinstall/", 
      keywords = ["ROS"],
      classifiers = [
        "Programming Language :: Python", 
        "License :: OSI Approved :: BSD License" ],
      description = "The installer for ROS", 
      long_description = """\
The installer for ROS
""",
      license = "BSD"
      )
